#include <boost/serialization/list.hpp>
#include <boost/serialization/string.hpp>
#include <boost/serialization/version.hpp>

#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>

#include <boost/archive/binary_iarchive.hpp>
#include <boost/archive/binary_oarchive.hpp>

#include <fstream>

using namespace boost::serialization;

class bus_route
{
public:
   int var1, var2;

private:
   friend class boost::serialization::access;

   template<class Archive>
   void serialize(Archive & ar, const unsigned int version)
   {
#if USE_VERSION == 1
      if(version > 0)
         ar & make_nvp("var2", var2);
#endif
      ar & make_nvp("var1", var1);
   }

public:
   bus_route() : var1(1), var2(2) {}
};

#if USE_VERSION == 1
BOOST_CLASS_VERSION(bus_route, 1)
#endif



void save_schedule(const bus_route &s, const char * filename){
    // make an archive
    std::ofstream ofs(filename);
    assert(ofs.good());

#if USE_BINARY == 1
    boost::archive::binary_oarchive oa(ofs);
#else
    boost::archive::xml_oarchive oa(ofs);
#endif

    oa << BOOST_SERIALIZATION_NVP(s);

    std::cout << "Saved, var1=" << s.var1 
#if USE_VERSION == 1
       << " var2=" << s.var2 
#endif
       << std::endl;
}

void
restore_schedule(bus_route &s, const char * filename)
{
    // open the archive
    std::ifstream ifs(filename);
    if (!ifs.good())
    {
       std::cout << "Could not open file to read: " << filename << std::endl;
       return;
    }

#if USE_BINARY == 1
    boost::archive::binary_iarchive ia(ifs);
#else
    boost::archive::xml_iarchive ia(ifs);
#endif

    // restore the schedule from the archive
    ia >> BOOST_SERIALIZATION_NVP(s);

    std::cout << "Restored, var1=" << s.var1 
#if USE_VERSION == 1
       << " var2=" << s.var2 
#endif
       << std::endl;
}



int main()
{
   bus_route route;

#if USE_BINARY == 1
   const char* filename = "bus_route.binary";
#else
   const char* filename = "bus_route.xml";
#endif

   restore_schedule(route, filename);

   // ensure that var1 is always 1, we never change it.
   // if its not 1, then the archive was incorrectly loaded.
   if (route.var1 != 1)
   {
      std::cerr << "FAIL! var1 should always be 1" << std::endl;
      exit(1);
   }

   save_schedule(route, filename);

   return 0;
}
